import subprocess
import os
import sys
from pathlib import Path
import pydub

def Help():
	print("python Convert_RIFF-WAV_to_NOPUS.py file.wav start_loop_in_samples end_loop_in_samples")
	print("If you don't want loop to be applied, write for both values 0")
	sys.exit()

if (len(sys.argv) != 4):
	Help()

try:
	int(sys.argv[2], 16)
except Exception as exc:
	print(exc)
	Help()

try:
	int(sys.argv[3], 16)
except Exception as exc:
	print(exc)
	Help()

start_loop = int(sys.argv[2], 16)
end_loop = int(sys.argv[3], 16)

if (start_loop == 0 and end_loop == 0):
	loop_flag = 0
else: loop_flag = 1

file = open(sys.argv[1], "rb")
if (file.read(4) != b"RIFF"):
	print("Not supported WAV file!")
	sys.exit()
WAVE_size = int.from_bytes(file.read(4), "little")
if (file.read(4) != b"WAVE"):
	print("Not supported WAV file!")
	sys.exit()
if (file.read(4) != b"fmt "):
	print("Not supported WAV file!")
	sys.exit()
if (int.from_bytes(file.read(4), "little") != 16):
	print("Not supported WAV file!")
	sys.exit()
if (int.from_bytes(file.read(2), "little") != 1):
	print("Not supported WAV file!")
	sys.exit() 
channel_count = int.from_bytes(file.read(2), "little")
sample_rate = int.from_bytes(file.read(4), "little")
byte_rate = int.from_bytes(file.read(4), "little")
sample_alignment = int.from_bytes(file.read(2), "little")
bit_depth = int.from_bytes(file.read(2), "little")
if (file.read(4) != b"data"):
	print("Not supported WAV file!")
	sys.exit()
data_bytes = int.from_bytes(file.read(4), "little")

sample_count = data_bytes // channel_count
sample_count = sample_count // (bit_depth // 8)

assert(start_loop < sample_count)
assert(end_loop <= sample_count)

file.seek(0)

orig_audio = pydub.AudioSegment.from_file(sys.argv[1], "wav")
orig_audio = orig_audio.set_frame_rate(48000)
orig_audio.export("%s.pcm" % Path(sys.argv[1]).stem, format="s16le")

catch = subprocess.run(["NXAenc.exe", "-f", "240", "-c", f"{channel_count}", "-i", "%s.pcm" % Path(sys.argv[1]).stem, "-o", "%s.nxopus" % Path(sys.argv[1]).stem], capture_output=True, text=False)
if (catch.stderr != b""):
	print(catch.stderr)
	sys.exit()

header_size = 0x40
os.makedirs("Converted", exist_ok=True)
pcm_file = open("%s.nxopus" % Path(sys.argv[1]).stem, "rb")
pcm_dump = pcm_file.read()
new_file = open("Converted/%s.wav" % Path(sys.argv[1]).stem, "wb+")
new_file.write(0x1.to_bytes(4, "little"))
new_file.write(channel_count.to_bytes(4, "little"))
new_file.write(len(pcm_dump).to_bytes(4, "little"))
new_file.write(header_size.to_bytes(4, "little"))
new_file.write(loop_flag.to_bytes(4, "little"))
new_file.write(start_loop.to_bytes(4, "little"))
if (end_loop == 0):
	new_file.write(sample_count.to_bytes(4, "little"))
else:
	new_file.write(end_loop.to_bytes(4, "little"))
new_file.write(sample_count.to_bytes(4, "little"))
new_file.write(0x4.to_bytes(4, "little"))
new_file.write(sample_rate.to_bytes(4, "little"))
new_file.write(b"\x00" * (header_size - (new_file.tell() % header_size)))
new_file.write(pcm_dump)
new_file.seek(0x4A)
new_file.write(0x80.to_bytes(2, "little"))
new_file.seek(0x68)
value = int.from_bytes(new_file.read(4), "big")
new_file.seek(0x5C)
new_file.write(value.to_bytes(4, "little"))
new_file.close()
pcm_file.close()

os.remove("%s.pcm" % Path(sys.argv[1]).stem)
os.remove("%s.nxopus" % Path(sys.argv[1]).stem)